/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.custom.saml.resolvers;

import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.model.User;
import com.liferay.saml.opensaml.integration.resolver.NameIdResolver;
import com.liferay.saml.persistence.model.SamlIdpSpConnection;
import com.liferay.saml.persistence.service.SamlIdpSpConnectionLocalService;

import java.util.Map;

import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;

/**
 * @author Tibor Lipusz
 * @author Stian Sigvartsen
 */
@Component(
	immediate = true,
	property = "service.ranking:Integer=" + (Integer.MIN_VALUE + 1),
	service = NameIdResolver.class
)
public class CustomNameIdResolver implements NameIdResolver {

	@Override
	public String resolve(
			User user, String entityId, String format, String spQualifierName,
			boolean allowCreate,
			NameIdResolverSAMLContext nameIdResolverSAMLContext)
		throws PortalException {

		if (_log.isDebugEnabled()) {
			_log.debug("Resolving");
		}

		// Implement custom resolver logic

		return _defaultNameIdResolver.resolve(
			user, entityId, format, spQualifierName, allowCreate,
			nameIdResolverSAMLContext);
	}

	@Reference(
		target = "(component.name=com.liferay.saml.opensaml.integration.internal.resolver.DefaultNameIdResolver)",
		unbind = "-"
	)
	public void setDefaultNameIdResolver(NameIdResolver nameIdResolver) {
		_defaultNameIdResolver = nameIdResolver;
	}

	@Activate
	protected void activate(Map<String, Object> properties) {
		if (_log.isDebugEnabled()) {
			_log.debug("Activated");

			_log.debug(
				"Default resolver: " + _defaultNameIdResolver.getClass());
		}
	}

	/**
	 * companyId can be retrieved from {@link User#getCompanyId()} or using
	 * {@link
	 * com.liferay.portal.kernel.security.auth.CompanyThreadLocal#getCompanyId(
	 * )}
	 */
	protected String getNameIdAttributeName(long companyId, String entityId)
		throws PortalException {

		SamlIdpSpConnection samlIdpSpConnection =
			_samlIdpSpConnectionLocalService.getSamlIdpSpConnection(
				companyId, entityId);

		return samlIdpSpConnection.getNameIdAttribute();
	}

	private static final Log _log = LogFactoryUtil.getLog(
		CustomNameIdResolver.class);

	private NameIdResolver _defaultNameIdResolver;

	@Reference
	private SamlIdpSpConnectionLocalService _samlIdpSpConnectionLocalService;

}